<?php
require_once(__DIR__.'/mediator.class.php');
require_once(__DIR__.'/endpoint_manager.class.php');
require_once(__DIR__.'/sparql.class.php');
require_once(__DIR__.'/book.class.php');

class Author {

	private $uri;
	private $firstname;
	private $lastname;
	private $birthYear;
	private $deathYear;
	private $books = null;

	public function __construct($uri) {
		$last = strrchr($uri, '/');
		$this->uri = substr($uri, 0, strlen($uri)-strlen($last)) . "/" . urlencode(substr($last, 1));
	}

	public function getURI($encoded=true) {
		return (($encoded) ? $this->uri : urldecode($this->uri));
	}

	public function setName($name) {
		$part = explode(", ", $name);
		if (isset($part[1])) {
			$this->setFirstName($part[1]);
			$this->setLastName($part[0]);
		} else {
			$this->setLastName($name);
		}
	}

	public function getName() {
		if (!isset($this->firstname) || !isset($this->lastname)) {
			$endpoint = Endpoint_Manager::getEndpoint($this->getURI(), false);
			$mediator = new Mediator($endpoint->getEndpoint());
			$names = $mediator->getAuthorName($this);
			if (!isset($this->firstname)) $this->firstname = $names[0];
			if (!isset($this->lastname)) $this->lastname = $names[1];
		}
		return $this->firstname . " " . $this->lastname;
	}

	public function setFirstName($firstname) {
		$this->firstname = $firstname;
	}

	public function getFirstName() {
		if (!isset($this->firstname)) {
			$endpoint = Endpoint_Manager::getEndpoint($this->getURI(), false);
			$mediator = new Mediator($endpoint->getEndpoint());
			$names = $mediator->getAuthorName($this);
			$this->firstname = $names[0];
			if (!isset($this->lastname)) $this->lastname = $names[1];
		}
		return $this->firstname;
	}

	public function setLastName($lastname) {
		$this->lastname = $lastname;
	}

	public function getLastName() {
		if (!isset($this->lastname)) {
			$endpoint = Endpoint_Manager::getEndpoint($this->getURI(), false);
			$mediator = new Mediator($endpoint->getEndpoint());
			$names = $mediator->getAuthorName($this);
			if (!isset($this->firstname)) $this->firstname = $names[0];
			$this->lastname = $names[1];
		}
		return $this->lastname;
	}
	
	public function setBirthYear($year) {
		$this->birthYear = $year;
	}
	
	public function setDeathYear($year) {
		if (is_null($year)) $year = false;
		$this->deathYear = $year;
	}

	public function getBirthYear() {
		if (!isset($this->birthYear)) {
			$endpoint = Endpoint_Manager::getEndpoint($this->getURI(), false);
			$mediator = new Mediator($endpoint->getEndpoint());
			$dates = $mediator->getAuthorYears($this);
			$this->setBirthYear($dates[0]);
			$this->setDeathYear($dates[1]);
		}
		return $this->birthYear;
	}

	public function getDeathYear() {
		if (!isset($this->deathYear)) {
			$endpoint = Endpoint_Manager::getEndpoint($this->getURI(), false);
			$mediator = new Mediator($endpoint->getEndpoint());
			$dates = $mediator->getAuthorYears($this);
			$this->setBirthYear($dates[0]);
			$this->setDeathYear($dates[1]);
		}
		return $this->deathYear;
	}

	public function getBooks($all=true) {
		if ($this->books == null) {
			$endpoint = Endpoint_Manager::getEndpoint($this->getURI(), false);
			$mediator = new Mediator($endpoint->getEndpoint());
			$this->books = $mediator->getBooksAuthor($this);
		}

		if ($all) return $this->books;
		else {
			$books = array();
			if (count($this->books) > 0) $books[] = $this->books[0];
			for ($i=1; $i<count($this->books); $i++) {
				for ($j=0; $j<count($books); $j++)
					if ($books[$j]->getName() == $this->books[$i]->getName())
						continue 2;
				$books[] = $this->books[$i];
			}
			return $books;
		}
	}

	public function getNumBooks() {
		if ($this->books == null) {
			$endpoint = Endpoint_Manager::getEndpoint($this->getURI(), false);
			$mediator = new Mediator($endpoint->getEndpoint());
			return $mediator->getNumBooksAuthor($this);
		} else return count($this->books);
	}

	public function getEquivalents() {
		$endpoint = Endpoint_Manager::getEndpoint($this->getURI(), false);
		$mediator = new Mediator($endpoint->getEndpoint());
		return $mediator->getEquivalents($this->getURI());
	}

}
?>
