<?php
require_once(__DIR__.'/sparql_wrapper.class.php');

class Cat_Sparql_Wrapper extends Sparql_Wrapper {

    public function getBooks(Criteria $crit, $type) {
		$query = "SELECT DISTINCT ".$this->bookVars()." WHERE {\n";
		$type = Endpoint_Manager::getWorkType($type);
		$query .= $this->bookTypeSparql($type);
		$query .= $this->bookSparql();
		$query .= $this->generateSubSparql($crit, false);
		$query .= "}\nORDER BY ?book_uri\n";
		//return $query;
		
		$sparql = new Sparql($this->endpoint->getEndpoint());
		$sparql->setPrefixes($this->endpoint->getPrefixes());
		$sparql->query($query);
		$books = array();
		$map = array_flip($sparql->getColNames());
		while ($row = $sparql->getNextRow()) {
			$book = new Book($row[$map["book_uri"]]);
			$author = new Author($row[$map["author_uri"]]);
			$book->setName($row[$map["book_name"]]);
			if (isset($map["author_first_name"]))
				$author->setFirstName($row[$map["author_first_name"]]);
			if (isset($map["author_last_name"]))
				$author->setLastName($row[$map["author_last_name"]]);
			if (isset($map["author_name"]))
				$author->setName($row[$map["author_name"]]);
			$book->setAuthor($author);
			$books[] = $book;
		}
		return $books;
	}
	
	protected function bookTypeSparql($type) {
		if ($this->endpoint->belongsWorkType($type, true)) {
			return "?book_uri a ".$type." .\n";
		} else {
			$target = $this->endpoint;
			$endpoints = Endpoint_Manager::getEndpoints();
			$source = null;
			foreach ($endpoints as $e) {
				if ($e->belongsWorkType($type, true))
					$source = $e;
			}
			if ($source === null) throw new Exception("Unknown ontology.");
			$rewritings = $this->rewrite($type, $source, $target);
			$rewritings = $this->leastLoss($rewritings);
			reset($rewritings);
			list($key, $value) = each($rewritings);
			return $this->generateSparql($rewritings, $key);
		}
	}
	
	protected function rewrite($term, $source, $target) {
		$result = array();
		$traverse = array($term);
		$visited = array();
		$synonym = array();
		while (count($traverse) > 0) {
			$t = array_pop($traverse);
			$intersec = false;
			foreach ($result as $k => $v) {
				if (is_array($v) &&array_search($t, $v) !== false
					&& $result[$k][0] == "INTERSEC") $intersec = true;
			}
			if (isset($result[$t])) continue;
			if ($target->belongsWorkType($t, true) && !$intersec) continue;
			$s = $this->getSynonym($t, $source, $target);
			if ($s !== false) {
				$result[$t] = array("SYNONYM", $s);
				continue;
			}
			$ceiling = $this->directSuper($t, $source, $target);
			$floor = $this->directSub($t, $source, $target);
			$traverse = array_merge($traverse, $ceiling, $floor);
			foreach ($ceiling as $k => $v) {
				if (isset($result[$v])) unset($ceiling[$k]);
			}
			array_values($ceiling);
			array_unshift($ceiling, "INTERSEC");
			foreach ($floor as $k => $v) {
				if (isset($result[$v])) unset($floor[$k]);
			}
			array_values($floor);
			array_unshift($floor, "UNION");
			if (count($ceiling) == 1 && count($floor) == 1)
				$result[$t] = false;
			elseif (count($ceiling) == 1)
				$result[$t] = $floor;
			elseif (count($floor) == 1)
				$result[$t] = $ceiling;
			else
				$result[$t] = array("OR", $ceiling, $floor);
		}
		return $result;
	}
	
	protected function getSynonym($term, $source, $target) {
		$result = array();
		$sparql_source = new Sparql($source->getEndpoint());
		$sparql_source->setPrefixes($source->getPrefixes());
		$sparql_source->setPrefixes($target->getPrefixes());
		$sparql_target = new Sparql($target->getEndpoint());
		$sparql_target->setPrefixes($target->getPrefixes());
		$sparql_target->setPrefixes($source->getPrefixes());
		$query = "SELECT DISTINCT ?a WHERE { { ?a owl:equivalentClass ".$term." .";
		$query .= "} UNION { ".$term." owl:equivalentClass ?a . } }";
		$sparql_target->query($query);
		while($row = $sparql_target->getNextRow())
			$result[] = $row[0];
		$sparql_source->query($query);
		while($row = $sparql_source->getNextRow())
			$result[] = $row[0];
		foreach ($result as $type)
			if ($target->belongsWorkType($type) !== false)
				return $target->belongsWorkType($type);
		return false;
	}
	
	public function directSuper($term, $source, $target) {
		$query = "SELECT DISTINCT ?a WHERE { ".$term." <http://www.w3.org/2000/01/rdf-schema#subClassOf> ?a . }";
		return $this->direct($query, $source, $target);
	}
	
	protected function directSub($term, $source, $target) {
		$query = "SELECT DISTINCT ?a WHERE { ?a <http://www.w3.org/2000/01/rdf-schema#subClassOf> ".$term." . }";
		return $this->direct($query, $source, $target);
	}
	
	protected function direct($query, $source, $target) {
		$result = array();
		$sparql_source = new Sparql($source->getEndpoint());
		$sparql_source->setPrefixes($source->getPrefixes());
		$sparql_source->setPrefixes($target->getPrefixes());
		$sparql_source->query($query);
		while($row = $sparql_source->getNextRow())
			if ($source->belongsWorkType($row[0]))
				$result[] = $source->belongsWorkType($row[0]);
		$sparql_target = new Sparql($target->getEndpoint());
		$sparql_target->setPrefixes($source->getPrefixes());
		$sparql_target->setPrefixes($target->getPrefixes());
		$sparql_target->query($query);
		while($row = $sparql_target->getNextRow())
			if ($target->belongsWorkType($row[0]))
				$result[] = $target->belongsWorkType($row[0]);
		return $result;
	}
	
	protected function generateSparql($rewritings, $type) {
		if (!isset($rewritings[$type])) return "?book_uri a ".$type." .\n";
		if (!$rewritings[$type]) return null;
		if ($rewritings[$type][0] == "SYNONYM") return "?book_uri a ".$rewritings[$type][1]." .\n";
		if ($rewritings[$type][0] == "INTERSEC") {
			if ($this->endpoint->belongsWorkType($rewritings[$type][1], true))
				return "?book_uri a ".$rewritings[$type][1]." .\nMINUS {\n".$this->generateSparql($rewritings, $rewritings[$type][1])."}\n";
			else return $this->generateSparql($rewritings, $rewritings[$type][1]);
		}
		$result = array();
		for ($i=1; $i<count($rewritings[$type]); $i++)
			$result[] = $this->generateSparql($rewritings, $rewritings[$type][$i]);
		$result = array_filter($result, function($a) { return !is_null($a); });
		return "{\n".implode("} UNION {\n", $result)."}\n";
	}
	
	protected function leastLoss($rewritings) {
		foreach (array_reverse($rewritings) as $k => $v) {
			if ($v[0] == "OR") {
				$s1 = $s2 = 0;
				for ($i=1; $i<count($v[1]); $i++)
					$s1 += $this->loss($rewritings, $v[1][$i]);
				for ($i=1; $i<count($v[2]); $i++)
					$s2 += $this->loss($rewritings, $v[2][$i]);
				if ($s1 > $s2)
					$rewritings[$k] = $v[2];
				else
					$rewritings[$k] = $v[1];
			}
		}
		return $rewritings;
	}
	
	protected function loss($rewritings, $type) {
		if (!isset($rewritings[$type])) return 0;
		if (!$rewritings[$type]) return 1;
		if ($rewritings[$type][0] == "SYNONYM") return 0;
		$sum = 0;
		for ($i=1; $i<count($rewritings[$type]); $i++)
			$sum += $this->loss($rewritings, $rewritings[$type][$i]);
		return $sum;
	}
	
	protected function bookSparql() {
		$sparql = "?book_uri ".$this->endpoint->getVocab("book_author")." ?author_uri ;\n";
		$sparql .= $this->endpoint->getVocab("book_name")." ?book_name .\n";
		$sparql .= "?author_uri a ".$this->endpoint->getVocab("type_author")." ;\n";
		if (!is_null($this->endpoint->getVocab("author_first_name")))
			$sparql .= $this->endpoint->getVocab("author_first_name")." ?author_first_name ;\n";
		if (!is_null($this->endpoint->getVocab("author_last_name")))
			$sparql .= $this->endpoint->getVocab("author_last_name")." ?author_last_name ;\n";
		$sparql .= $this->endpoint->getVocab("author_name")." ?author_name .\n";
		return $sparql;
	}
	
}
?>
