<?php

class Criteria {
	
	private $globalmatch;
	private $searchgroup = array();
	private $match = array();
	
	public function __construct($p=null) {
		if ($p == null) return;
		
		if (!isset($p['globalmatch'])) $p['globalmatch'] = "AND";
		$this->setGlobalMatch($p['globalmatch']);
		unset($p['globalmatch']);
		
		// get the group ids
		$groups = array();
		foreach ($p as $k => $v) {
			$part = explode("_", $k);
			if (isset($part[2]) && is_numeric($part[1])) $groups[] = $part[0];
		}
		$groups = array_flip(array_values(array_unique($groups)));
		
		// create the search groups
		foreach ($groups as $k => $v) {
			if (isset($p[$k."_match"])) {
				$this->newSearchGroup($p[$k."_match"]);
				unset($p[$k."_match"]);
			}
			else $this->newSearchGroup();
		}
		
		// populate them
		while (count($p) > 0) {
			$v = reset($p);
			$k = key($p);
			$part = explode("_", $k);
			if (!isset($part[1])) {
				unset($p[$k]);
				continue;
			}
			$f = $part[0]."_".$part[1]."_";
			$hyp = explode("-", $p[$f."1"]);
			$this->newSearchTermIn($groups[$part[0]], $hyp[0], $p[$f."3"], (isset($hyp[1])) ? $hyp[1] : null);
			unset($p[$f."1"]);
			unset($p[$f."3"]);
		}
	}
	
	public function setGlobalMatch($match) {
		if ($match != "AND" && $match != "OR") throw new Exception("Incorrect global match option", 1001);
		$this->globalmatch = $match;
	}
	
	public function newSearchGroup($localmatch="AND") {
		if ($localmatch != "AND" && $localmatch != "OR") throw new Exception("Incorrect match option in search group", 1002);
		$this->match[] = $localmatch;
		$this->searchgroup[] = array();
	}
	
	public function newSearchTermIn($n, $type, $text1, $text2=null) {
		if ($n >= count($this->searchgroup) || $n < 0) throw new Exception("Search group ".$n." does not exist", 1003);
		if ($type == "author") {
			if (is_null($text2)) throw new Exception("Author search field needs two parameters", 1008);
			if ($text2 != "is" && $text2 != "starts" && $text2 != "contains")
				throw new Exception("Incorrect option ".$text2." in Author search field", 1009);
			$this->searchgroup[$n][] = array($type, $text1, $text2);
		} elseif ($type == "book") {
			$this->searchgroup[$n][] = array($type, $text1);
		} else {
			throw new Exception("Incorrect type ".$type, 1004);
		}
	}
	
	public function newSearchTerm($type, $text1, $text2=null) {
		if (count($this->searchgroup) == 0) throw new Exception("There are no search groups", 1005);
		$this->newSearchTermIn(count($this->searchgroup)-1, $type, $text1, $text2);
	}
	
	public function getCriteria() {
		if (count($this->searchgroup) > 1 && $this->globalmatch == null) throw new Exception("Empty global match option", 1006);
		$result = $this->searchgroup;
		for ($i=0; $i<count($result); $i++) {
			if (count($result[$i]) == 0) throw new Exception("Empty search group ".$i, 1007);
			$result[$i]["match"] = $this->match[$i];
		}
		if (count($this->searchgroup) != 0) 
			$result["match"] = ($this->globalmatch != null) ? $this->globalmatch : "AND";
		return $result;
	}
	
}
?>
