<?php
//require_once(dirname(__FILE__).'/sparql.class.php');

class Endpoint {

	private $name;
	private $endpoint;
	private $uri;
	private $vocab = array();
	private $types = array();
	private $prefixes = array();

	public function __construct($data=null) {
		if ($data != null) {
			if (!$this->isWellFormed($data)) throw new Exception("Incorrect endpoint data", 92);
			$this->setEndpoint($data["endpoint"]);
			$this->setName($data["name"]);
			$this->setResource($data["resource"]);
			if (isset($data["bifContains"]) && $data["bifContains"]) $this->setbifContains();
			for ($i=0; $i<count($data["vocab"]); $i++)
				$this->addVocab($data["vocab"][$i]["type"], $data["vocab"][$i]["word"]);
			for ($i=0; $i<count($data["work"]); $i++)
				$this->addWorkType($data["work"][$i]["type"]);
			for ($i=0; $i<count($data["pref"]); $i++)
				$this->addPrefix($data["pref"][$i]["prefix"], $data["pref"][$i]["iri"]);
			$this->addPrefix("owl", "http://www.w3.org/2002/07/owl#");
			$this->addPrefix("local", $data["resource"]);
		}	
	}

	private function isWellFormed($data) {
		$bool = true;
		$bool &= isset($data["name"]);
		$bool &= isset($data["endpoint"]);
		$bool &= isset($data["resource"]);
		$bool &= isset($data["vocab"]);
		$bool &= isset($data["work"]);
		return $bool;
	}

	public function setEndpoint($endpoint) {
		//$sparql = new Sparql($endpoint);
		//if(!$sparql->isAlive())
		//	throw new Exception("Not alive", 91);
		$this->endpoint = $endpoint;
	}

	public function getEndpoint() {
		return $this->endpoint;
	}

	public function setName($name) {
		$this->name = $name;
	}

	public function getName() {
		return $this->name;
	}

	public function getShortName() {
		$part = explode(" (", $this->name);
		return $part[0];
	}

	public function setResource($uri) {
		$this->uri = $uri;
	}

	public function belongs($uri) {
		return (substr($uri, 0, strlen($this->uri)) == $this->uri);
	}

	public function addVocab($type, $w) {
		$this->vocab[$type] = $w;
	}

	public function getVocab($type) {
		return (isset($this->vocab[$type])) ? $this->vocab[$type] : null;
	}

	public function addWorkType($type) {
		$this->types[] = $type;
	}

	public function getWorkTypes() {
		return $this->types;
	}
	
	public function belongsWorkType($uri, $prefix=false) {
		if (!$prefix) {
			foreach ($this->getPrefixes() as $k => $v)
				$trans[$k.":"] = $v;
			foreach ($this->types as $type)
				if (strtr($type, $trans) == $uri)
					return $type;
			return false;
		} else return array_search($uri, $this->types) !== false;
	}

	public function addPrefix($prefix, $iri) {
		$this->prefixes[$prefix] = $iri;
	}

	public function getPrefixes() {
		return $this->prefixes;
	}

}
?>
