<?php
require_once(__DIR__.'/../config.php');
require_once(__DIR__.'/endpoint.class.php');
require_once(__DIR__.'/mediator.class.php');
require_once(__DIR__.'/local_dataset.class.php');

class Endpoint_Manager {

	static private $instanced = false;
	static private $endpoints = array();
	static private $local = null;
	static private $types = array();

	private function __construct() {
	}

	private static function construct() {
		if (self::$instanced) return;
		self::$instanced = true;

		$folder = dirname(__FILE__)."/../endpoints/";
		$all = scandir($folder);
		// lambda function + filter + reindex
		$files = array_values(array_filter($all, function ($a) { return substr($a, -5) == ".json"; }));
		foreach ($files as $f) {
			try {
				$endpointdata = json_decode(file_get_contents($folder.$f), true);
				$endpointdata = ($endpointdata != null) ? $endpointdata : true;
				$endpoint = new Endpoint($endpointdata);
				self::register($endpoint);
				if (count($endpoint->getWorkTypes()) > 1)
					self::$types = array_merge(self::$types, $endpoint->getWorkTypes());
			} catch (Exception $e) {}
		}

	}

	private static function register($endpoint) {
		self::$endpoints[] = $endpoint;
	}

	public static function getEndpoint($endpoint, $direct=true) {
		self::construct();
		if ($direct) {
			for ($i=0; $i<count(self::$endpoints); $i++)
				if (self::$endpoints[$i]->getEndpoint() == $endpoint)
					return self::$endpoints[$i];
		} else {
			for ($i=0; $i<count(self::$endpoints); $i++)
				if (self::$endpoints[$i]->belongs($endpoint))
					return self::$endpoints[$i];
		}
		throw new Exception("Not an endpoint", 93);
	}
	
	public static function local() {
		self::construct();
		if (!is_null(self::$local)) return self::$local;
		global $config;
		if (!isset($config["endpoint"]) || strlen($config["endpoint"]) == 0)
			return (self::$local = false);
		$endpoints = self::getEndpoints();
		for ($i=0; $i<count($endpoints); $i++)
			if ($endpoints[$i]->getEndpoint() == $config["endpoint"]) {
				self::$local = new Local_Dataset($config['endpoint'], $config['user'], $config['pass']);
				return self::$local;
			}
		throw new Exception($config["endpoint"]." is not a defined endpoint", 95);
	}

	public static function getEndpoints($full=true) {
		self::construct();
		if ($full) return self::$endpoints;
		return array_map(function ($a) { return array($a->getName(), $a->getEndpoint()); }, self::$endpoints); // short and sweet
	}

	public static function belongs($uri, $full=true) {
		self::construct();
		foreach (self::$endpoints as $e) {
			if ($e->belongs($uri)) {
				if ($full) return $e;
				return $e->getEndpoint();
			}
		}
		return false;
	}
	
	public static function exists($uri) {
		self::construct();
		$e = self::belongs($uri, false);
		if (!$e) return false;
		$mediator = new Mediator($e);
		return $mediator->exists($uri);
	}
	
	public static function getWorkTypes($processed=false) {
		self::construct();
		if (!$processed) return self::$types;
		// remove prefixes and duplicates, and sort them
		$result = array_unique(array_map(function($a) { return substr(strstr($a, ':'), 1); }, self::$types));
		sort($result);
		return $result;
	}
	
	public static function getWorkType($match) {
		foreach (self::$types as $t)
			if (substr(strstr($t, ':'), 1) == $match)
				return $t;
		return null;
	}

}

?>
