<?php
require_once(__DIR__.'/author.class.php');
require_once(__DIR__.'/mediator.class.php');

class Local_Dataset {

	private $endpoint;
	private $user;
	private $pass;
	private $root;

	public function __construct($endpoint, $user, $pass) {
		$this->endpoint = Endpoint_Manager::getEndpoint($endpoint);
		$this->user = $user;
		$this->pass = $pass;
		$this->root = substr($this->endpoint->getEndpoint(), 0, strrpos($this->endpoint->getEndpoint(), '/'));
	}
	
	public function insertData($data) {
		$mediator = new Mediator($this->endpoint->getEndpoint());
		$data = $this->processBooks($data);
		$prefixes = $this->endpoint->getPrefixes();
		$triples = $previds = array();
		$tildes = array(" ", "Á", "á", "É", "é", "Í", "í", "Ó", "ó", "Ú", "ú", "Ñ", "ñ", "Ç", "ç");
		$wotildes = array("_", "A", "a", "E", "e", "I", "i", "O", "o", "U", "u", "N", "n", "C", "c");
		$id = str_replace($tildes, $wotildes, $data['firstname']."_".$data['lastname']);
		$id = "local:".preg_replace("/[^a-z_-]/i", "", iconv('UTF-8', 'ASCII//TRANSLIT//IGNORE', $id));
		$i = 1;
		while ($this->exists(str_replace("local:", $prefixes['local'], $id))) {
			if ($i == 1) $origid = $id;
			$id = $origid."_".++$i;
		}
		$triples[] = $this->makeTriple($id, "a", $this->endpoint->getVocab("type_author"), false);
		$triples[] = $this->makeTriple($id, $this->endpoint->getVocab("author_name"), $data['lastname'].", ".$data['firstname']);
		$triples[] = $this->makeTriple($id, $this->endpoint->getVocab("author_first_name"), $data['firstname']);
		$triples[] = $this->makeTriple($id, $this->endpoint->getVocab("author_last_name"), $data['lastname']);
		if (strlen($data['birthyear']) > 0) $triples[] = $this->makeTriple($id, $this->endpoint->getVocab("author_birth"), $data['birthyear']);
		if (strlen($data['deathyear']) > 0) $triples[] = $this->makeTriple($id, $this->endpoint->getVocab("author_death"), $data['deathyear']);
		
		$author = new Author($data['uri']);
		$equiv = $author->getEquivalents();
		$equivuris = array_map(function ($a) { return $a[0]; }, $equiv);
		$equivuris[] = $data['uri'];
		if ($this->isThere($equivuris, true)) throw new Exception("The author is already in the dataset");
		$triples[] = $this->makeTriple($id, "owl:sameAs", "<".$data['uri'].">", false);
		foreach ($equiv as $sa)
			$triples[] = $this->makeTriple($id, "owl:sameAs", "<".$sa.">", false);
		
		foreach($data['books'] as $book) {
			$bookid = substr(str_replace($tildes, $wotildes, $book[0]), 0, 50);
			$bookid = "local:".preg_replace("/[^a-z_-]/i", "", iconv('UTF-8', 'ASCII//TRANSLIT//IGNORE', $bookid));
			$i = 1;
			while (array_search($bookid, $previds) !== false || $this->exists(str_replace("local:", $prefixes['local'], $bookid))) {
				if ($i == 1) $origbookid = $bookid;
				$bookid = $origbookid."_".++$i;
			}
			$previds[] = $bookid;
			$terms = array($book[2]);
			while (count($terms) > 0) {
				$t = array_pop($terms);
				$triples[] = $this->makeTriple($bookid, "a", $t, false);
				$terms = array_merge($terms, array_unique(@$mediator->directSuper($t, $this->endpoint, $this->endpoint)));
			}
			$triples[] = $this->makeTriple($bookid, $this->endpoint->getVocab("book_name"), $book[0]);
			$triples[] = $this->makeTriple($bookid, $this->endpoint->getVocab("book_author"), $id, false);
			if (!is_null($book[1])) $triples[] = $this->makeTriple($bookid, "owl:sameAs", "<".$book[1].">", false);
		}
		
		$query = $this->generateInsertQuery($prefixes, $triples);
//		return $query;

		$webdav = $this->root."/DAV/home/".$this->user."/rdf_sink/rdf";
		$this->makeQuery($webdav, "add", $query);
		$this->makeQuery($webdav, "delete");
		return str_replace("local:", $prefixes['local'], $id);
	}
	
	private function processBooks($data) {
		$keys = array();
		if (isset($data['newbook'])) unset($data['newbook']);
		reset($data);
		while (list($k, $v) = each($data)) {
			$data[$k] = strip_tags($v);
			if (substr($k, 0, 9) == "bookname_") {
				$keys[] = substr($k, 9);
			}
		}
		$map = array_flip($keys);
		for ($i=0; $i<count($keys); $i++) {
			$bookname = strip_tags($data['bookname_'.$keys[$i]]);
			unset($data['bookname_'.$keys[$i]]);
			$booktype = strip_tags($data['booktype_'.$keys[$i]]);
			unset($data['booktype_'.$keys[$i]]);
			if (isset($data['bookuri_'.$keys[$i]])) {
				$bookuri = $data['bookuri_'.$keys[$i]];
				unset($data['bookuri_'.$keys[$i]]);
			} else $bookuri = null;
			$data['books'][] = array($bookname, $bookuri, $booktype);
		}
		return $data;
	}
	
	private function makeTriple($one, $two, $three, $literal=true) {
		if ($literal) $three = "'".str_replace("'", "\'", $three)."'";
		return $one." ".$two." ".$three;
	}
	
	private function generateInsertQuery($prefixes, $triples) {
		$query = "";
		foreach ($prefixes as $p => $u) {
			$query .= "PREFIX ".$p.": <".$u.">\n";
		}
		$query .= "INSERT INTO <".$this->root."/littera> {\n";
		for ($i=0; $i<count($triples); $i++) {
			$query .= $triples[$i]." .\n";
		}
		$query .= "}";
		return $query;
	}
	
	private function makeQuery($url, $type, $query=null) {
		$ch = curl_init($url);
		if ($type == "add") {
			curl_setopt($ch, CURLOPT_POSTFIELDS, $query);
			curl_setopt($ch, CURLOPT_HTTPHEADER, array ("Content-Type: application/sparql-query"));
		} elseif ($type == "delete") {
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "DELETE");
		} else {
			throw new Exception("Unknown parameter ".$type);
		}
		curl_setopt($ch, CURLOPT_USERPWD, $this->user.":".$this->pass);
		curl_setopt($ch, CURLOPT_TIMEOUT, 3);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		$output = curl_exec($ch);
		$info = curl_getinfo($ch);

		if (curl_errno($ch))
			throw new Exception("Curl error: ".curl_error($ch), curl_errno($ch));

		if ($info['http_code'] < 200 || $info['http_code'] >= 300)
			throw new Exception($output, $info['http_code']);
	}
	
	public function isThere($uri, $full=false) {
		$mediator = new Mediator($this->endpoint->getEndpoint());
		if (!$full) $uri = array($uri);
		return $mediator->hasEquivalentTo($uri);
	}
	
	public function belongs($uri) {
		return $this->endpoint->belongs($uri);
	}
	
	public function exists($uri) {
		return ($this->belongs($uri) && Endpoint_Manager::exists($uri));
	}
	
	public function getEndpoint($full=false) {
		if (!$full) return $this->endpoint->getEndpoint();
		return $this->endpoint;
	}
	
	public function insertBook($title, $author, $type) {
		$mediator = new Mediator($this->endpoint->getEndpoint());
		if (!$this->exists($author)) return "false";
		$prefixes = $this->endpoint->getPrefixes();
		$tildes = array(" ", "Á", "á", "É", "é", "Í", "í", "Ó", "ó", "Ú", "ú", "Ñ", "ñ", "Ç", "ç");
		$wotildes = array("_", "A", "a", "E", "e", "I", "i", "O", "o", "U", "u", "N", "n", "C", "c");
		$bookid = substr(str_replace($tildes, $wotildes, $title), 0, 50);
		$bookid = "local:".preg_replace("/[^a-z_-]/i", "", iconv('UTF-8', 'ASCII//TRANSLIT//IGNORE', $bookid));
		$i = 1;
		while ($this->exists(str_replace("local:", $prefixes['local'], $bookid))) {
			if ($i == 1) $origbookid = $bookid;
			$bookid = $origbookid."_".++$i;
		}
		$previds[] = $bookid;
		$terms = array($type);
		while (count($terms) > 0) {
			$t = array_pop($terms);
			$triples[] = $this->makeTriple($bookid, "a", $t, false);
			$terms = array_merge($terms, array_unique(@$mediator->directSuper($t, $this->endpoint, $this->endpoint)));
		}
		$triples[] = $this->makeTriple($bookid, $this->endpoint->getVocab("book_name"), $title);
		$triples[] = $this->makeTriple($bookid, $this->endpoint->getVocab("book_author"), "<".$author.">", false);
		
		$query = $this->generateInsertQuery($prefixes, $triples);
		
		$webdav = $this->root."/DAV/home/".$this->user."/rdf_sink/rdf";
		$this->makeQuery($webdav, "add", $query);
		$this->makeQuery($webdav, "delete");
		return str_replace("local:", $prefixes['local'], $bookid);
	}

}
?>
