<?php

class Sparql {

	private $endpoint;
	private $prefixes = array();
	private $result = null;
	private $current;

	public function __construct ($endpoint) {
		$this->endpoint = $endpoint;
	}

	public function isAlive() {
		try {
			$this->query("select ?Concept where {[] a ?Concept} LIMIT 1", 2);
			return true;
		} catch (Exception $e) {
			return false;
		}
	}

	public function addPrefix($prefix, $iri) {
		$this->prefixes[$prefix] = $iri;
	}

	public function setPrefixes($prefixes) {
		foreach ($prefixes as $prefix => $iri)
			$this->prefixes[$prefix] = $iri;
	}

	private function getPrefixes() {
		$p = null;
		foreach ($this->prefixes as $prefix => $iri)
			$p .= "PREFIX $prefix: <$iri>\n";
		return $p;
	}

	public function query($query, $timeout=null) {
		$this->resetResults();
		$url = $this->endpoint . "?query=" . urlencode($this->getPrefixes().$query);

		$ch = curl_init($url);
		if ($timeout != null) curl_setopt($ch, CURLOPT_TIMEOUT, $timeout);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array ("Accept: application/sparql-results+json"));
		$output = curl_exec($ch);
		$info = curl_getinfo($ch);

		if (curl_errno($ch)) // Error codes: 1-88
			throw new Exception("Curl error: ".curl_error($ch), curl_errno($ch));

		if ($output === false) // Error code 94
			throw new Exception("Timeout", 94);

		if ($output === '') // Error code 90
			throw new Exception("URL returned no data", 90);

		if ($info['http_code'] != 200) // Error codes: 100-599
			throw new Exception($output, $info['http_code']);

		curl_close($ch);
//$this->result = $output; return;
		$this->result = json_decode($output, true);

		if ($this->result == null) // Error code 95
			throw new Exception("JSON error: ".$output, 95);

		return;
	}

	public function getColNames() {
		return $this->result['head']['vars'];
	}

	public function getNumCols() {
		return count($this->getColNames());
	}

	public function getNumRows() {
		return count($this->result['results']['bindings']);
	}

	public function getNthRow($n, $full=false) {
		if ($n < 0 || $n >= $this->getNumRows()) throw new Exception("Out of range: Requested ".$n." and range is [0,".$this->getNumRows()."]", 98);
		if ($full) return $this->result['results']['bindings'][$n];
		else {
			$cols = $this->getColNames();
			for ($i=0; $i<$this->getNumCols(); $i++)
				if (isset($this->result['results']['bindings'][$n][$cols[$i]]['value']))
					$row[$i] = $this->result['results']['bindings'][$n][$cols[$i]]['value'];
				else
					$row[$i] = null;
			return $row;
		}
	}

	public function getNextRow($full=false) {
		try {
			return $this->getNthRow($this->current++, $full);
		} catch (Exception $e) {
			return false;
		}
	}

	public function restartResults() {
		$this->current = 0;
	}

	public function resetResults() {
		$this->result = null;
		$this->current = 0;
	}

	public function getScalar() {
		if ($this->getNumCols() != 1 or $this->getNumRows() != 1)
			throw new Exception("Not scalar: ".$this->getNumCols()." cols and ".$this->getNumRows()." rows", 99);
		$colnames = $this->getColNames();
		return $this->result['results']['bindings'][0][$colnames[0]]['value'];
	}
}

?>
