<?php
require_once(__DIR__.'/wrapper.interface.php');
require_once(__DIR__.'/endpoint_manager.class.php');
require_once(__DIR__.'/sparql.class.php');
require_once(__DIR__.'/author.class.php');
require_once(__DIR__.'/book.class.php');

abstract class Sparql_Wrapper implements Wrapper {
	
	protected $endpoint;
	
	public function __construct($endpoint) {
		$this->endpoint = Endpoint_Manager::getEndpoint($endpoint);
	}
	
    public function getAuthors(Criteria $crit) {
		$query = "SELECT DISTINCT ".$this->authorVars()." WHERE {\n";
		$query .= $this->authorSparql();
		$query .= $this->generateSubSparql($crit);
		$query .= "}\nORDER BY ?author_uri\n";
		//return $query;
		
		$sparql = new Sparql($this->endpoint->getEndpoint());
		$sparql->setPrefixes($this->endpoint->getPrefixes());
		$sparql->query($query);
		$authors = array();
		$map = array_flip($sparql->getColNames());
		$i = 0;
		while ($row = $sparql->getNextRow()) {
			if ($i>0 && $authors[$i-1]->getURI(false) == $row[0]) {
				if (!$authors[$i-1]->getFirstName()) {
					if (isset($map["author_first_name"]))
						$authors[$i-1]->setFirstName($row[$map["author_first_name"]]);
					if (isset($map["author_last_name"]))
						$authors[$i-1]->setLastName($row[$map["author_last_name"]]);
					if (isset($map["author_name"]))
						$authors[$i-1]->setName($row[$map["author_name"]]);
				}
				continue;
			}
			$author = new Author($row[$map["author_uri"]]);
			if (isset($map["author_first_name"]))
				$author->setFirstName($row[$map["author_first_name"]]);
			if (isset($map["author_last_name"]))
				$author->setLastName($row[$map["author_last_name"]]);
			if (isset($map["author_name"]))
				$author->setName($row[$map["author_name"]]);
			if (isset($map["author_birth"])) {
				$part = explode("-", $row[$map["author_birth"]]);
				$year = ($row[$map["author_birth"]] != null) ? $part[0] : false;
				$author->setBirthYear($year);
			}
			if (isset($map["author_death"])) {
				$part = explode("-", $row[$map["author_death"]]);
				$year = ($row[$map["author_death"]] != null) ? $part[0] : false;
				$author->setDeathYear($year);
			}
			if (isset($map["author_birth_death"])) {
				$part = explode("-", $row[$map["author_birth_death"]]);
				$birth = (isset($part[0]) && $part[0] != null) ? $part[0] : false;
				$death = (isset($part[1]) && $part[1] != null) ? $part[1] : false;
				$author->setBirthYear($birth);
				$author->setDeathYear($death);
			}
			$authors[$i++] = $author;
		}
		return $authors;
	}
	
	protected function authorVars() {
		$vars = "?author_uri ";
		if ($this->endpoint->getVocab("author_first_name") != null)
			$vars .= "?author_first_name ";
		if ($this->endpoint->getVocab("author_last_name") != null)
			$vars .= "?author_last_name ";
		if ($vars == "?author_uri ")
			$vars .= "?author_name ";
		if ($this->endpoint->getVocab("author_birth") != null)
			$vars .= "?author_birth ";
		if ($this->endpoint->getVocab("author_death") != null)
			$vars .= "?author_death ";
		if ($this->endpoint->getVocab("author_birth_death") != null)
			$vars .= "?author_birth_death ";
		return $vars;
	}
	
	protected function authorSparql() {
		$sparql = "?author_uri a ".$this->endpoint->getVocab("type_author")." ;\n";
		if (!is_null($this->endpoint->getVocab("author_first_name")))
			$sparql .= $this->endpoint->getVocab("author_first_name")." ?author_first_name ;\n";
		if (!is_null($this->endpoint->getVocab("author_last_name")))
			$sparql .= $this->endpoint->getVocab("author_last_name")." ?author_last_name ;\n";
		$sparql .= $this->endpoint->getVocab("author_name")." ?author_name .\n";
		$optional = null;
		if (!is_null($this->endpoint->getVocab("author_birth")))
			$optional .= "?author_uri ".$this->endpoint->getVocab("author_birth")." ?author_birth .\n";
		if (!is_null($this->endpoint->getVocab("author_death")))
			$optional .= "?author_uri ".$this->endpoint->getVocab("author_death")." ?author_death .\n";
		if (!is_null($this->endpoint->getVocab("author_birth_death")))
			$optional .= "?author_uri ".$this->endpoint->getVocab("author_birth_death")." ?author_birth_death .\n";
		if (!is_null($optional)) $sparql .= "OPTIONAL {\n".$optional."}\n";
		return $sparql;
	}
	
	protected function buildCondition($cond, $author=true) {
		if ($cond[0] == "author") {
			$name = (!is_null($this->endpoint->getVocab("author_last_name"))) ? $this->endpoint->getVocab("author_last_name") : $this->endpoint->getVocab("author_name");
			if ($cond[2] == "is") {
				$sub = "{ ?author_uri ".$name." '".$cond[1]."' .\n } UNION { ?author_uri ".$name." '".$cond[1]."'@en .\n }";
				//$sub = "?author_uri ".$name." '".$cond[1]."' .\n";
			} elseif ($cond[2] == "starts") {
				$sub = "?author_uri ".$name." ?name .\n";
				$sub .= "FILTER regex(?name, '^".$cond[1]."([ ,]|$)') .\n";
			} elseif ($cond[2] == "contains") {
				$sub = "?author_uri ".$this->endpoint->getVocab("author_name")." ?name .\n";
				$sub .= "FILTER regex(?name, '".$cond[1]."') .\n";
			}
		} elseif ($cond[0] == "book") {
			if ($author) {
				$sub = "?book_uri a ".$this->endpoint->getVocab("type_book")." ;\n";
				$sub .= $this->endpoint->getVocab("book_author")." ?author_uri ;\n";
				$sub .= $this->endpoint->getVocab("book_name")." ?book_name .\n";
				$sub .= "FILTER regex(?book_name, '".$cond[1]."', 'i') .\n";
			} else {
				$sub = "?book_uri ".$this->endpoint->getVocab("book_name")." ?book_name .\n";
				$sub .= "FILTER regex(?book_name, '".$cond[1]."', 'i') .\n";
			}
		} else {
			throw new Exception("Incorrect option ".$cond[0]);
		}
		return $sub;
	}
	
	protected function generateSubSparql(Criteria $crit, $author=true) {
		$cond = $crit->getCriteria();
		$subquery = array();
		for ($i=0; $i<count($cond)-1; $i++) {
			$sub = array();
			for ($j=0; $j<count($cond[$i])-1; $j++) {
					$sub[] = $this->buildCondition($cond[$i][$j], $author);
			}
			if ($cond[$i]['match'] == "OR")
				$subquery[] = "{\n".implode("} UNION {\n", $sub)."}\n";
			else
				$subquery[] = implode($sub);
		}
		if ($cond['match'] == "OR")
			return "{\n".implode("} UNION {\n", $subquery)."}\n";
		else
			return implode($subquery);
	}
	
    abstract public function getBooks(Criteria $crit, $type);
    
	public function getNumBooksAuthor($author) {
		$sparql = new Sparql($this->endpoint->getEndpoint());
		$sparql->setPrefixes($this->endpoint->getPrefixes());
		$query = "SELECT COUNT(?book) WHERE {";
		$query .= "?book ".$this->endpoint->getVocab("book_author")." <".$author->getURI()."> }";
		$sparql->query($query);
		return $sparql->getScalar();
	}
	
	public function getBooksAuthor($author) {
		$sparql = new Sparql($this->endpoint->getEndpoint());
		$sparql->setPrefixes($this->endpoint->getPrefixes());
		$query = "SELECT ?book_uri ?book_name { ";
		$query .= "?book_uri ".$this->endpoint->getVocab("book_author")." <".$author->getURI()."> ; ";
		$query .= $this->endpoint->getVocab("book_name")." ?book_name }";
		$sparql->query($query);
		$books = array();
		$i = 0;
		while ($row = $sparql->getNextRow()) {
			if ($i>0 && $books[$i-1]->getURI() == $row[0]) {
				$books[$i-1]->setName($row[1], true);
			} else {
				$book = new Book($row[0]);
				$book->setName($row[1]);
				$book->setAuthor($author);
				$books[$i++] = $book;
			}
		}
		return $books;
	}
	
	public function getAuthorYears($author) {
		$sparql = new Sparql($this->endpoint->getEndpoint());
		$sparql->setPrefixes($this->endpoint->getPrefixes());
		if (!is_null($this->endpoint->getVocab("author_birth")) && !is_null($this->endpoint->getVocab("author_death"))) {
			$query = "SELECT ?author_birth ?author_death { ";
			$query .= "<".$author->getURI()."> ".$this->endpoint->getVocab("author_birth")." ?author_birth .\n";
			$query .= "OPTIONAL { <".$author->getURI()."> ".$this->endpoint->getVocab("author_death")." ?author_death }}";
			try {
				$sparql->query($query);
				$row = $sparql->getNextRow();
				$birth = explode('-', $row[0]);
				$death = explode('-', $row[1]);
				return array($birth[0], $death[0]);
			} catch (Exception $e) { }
		} elseif ($this->endpoint->getVocab("author_birth_death") !== null) {
			$query = "SELECT ?author_birth_death { ";
			$query .= "<".$author->getURI()."> ".$this->endpoint->getVocab("author_birth_death")." ?author_birth_death }";
			try {
				$sparql->query($query);
				return explode('-', $sparql->getScalar());
			} catch (Exception $e) { }
		}
	}
	
	public function getAuthorName($author) {
		$sparql = new Sparql($this->endpoint->getEndpoint());
		$sparql->setPrefixes($this->endpoint->getPrefixes());
		if (is_null($this->endpoint->getVocab("author_first_name")) || is_null($this->endpoint->getVocab("author_last_name"))) {
			$query = "SELECT ?author_name { ";
			$query .= "<".$author->getURI()."> ".$this->endpoint->getVocab("author_name")." ?author_name }";
			try {
				$sparql->query($query);
				$row[0] = $sparql->getScalar();
			} catch (Exception $e) {
				while ($row = $sparql->getNextRow()) {
					if (strstr($row[0], ',')) break;
				}
			}
			$part = explode(", ", $row[0]);
			return array(((isset($part[1])) ? $part[1] : false), $part[0]);
		} else {
			$query = "SELECT ?author_first_name ?author_last_name WHERE { ";
			$query .= "<".$author->getURI()."> ".$this->endpoint->getVocab("author_first_name")." ?author_first_name ;";
			$query .= $this->endpoint->getVocab("author_last_name")." ?author_last_name }";
			try {
				$sparql->query($query);
				$row = $sparql->getNextRow();
			} catch (Exception $e) { }
			return array($row[0], $row[1]);
		}
	}
	
	protected function bookVars() {
		$vars = "?book_uri ?book_name ?author_uri ";
		if ($this->endpoint->getVocab("author_first_name") != null)
			$vars .= "?author_first_name ";
		if ($this->endpoint->getVocab("author_last_name") != null)
			$vars .= "?author_last_name ";
		if ($vars == "?book_uri ?book_name ?author_uri ")
			$vars .= "?author_name ";
		return $vars;
	}
	
	public function getDirectEquivalent($uris) {
		$sparql = new Sparql($this->endpoint->getEndpoint());
		$sparql->setPrefixes($this->endpoint->getPrefixes());
		try {
			$u = array();
			foreach ($uris as $uri) {
				$u[] = "?a owl:sameAs <".$uri."> .";
			}
			$query = "SELECT ?a WHERE {\n{\n".implode("\n} UNION {\n", $u)."\n}\n}";
			$sparql->query($query);
			if ($sparql->getNumRows() == 0) return false;
			$row = $sparql->getNextRow();
			return $row[0];
		} catch (Exception $ex) {
			if ($sparql->isAlive()) throw $ex;
			// otherwise, do nothing, since it surely is temporarily down
		}
	}
	
	public function exists($uri) {
		$sparql = new Sparql($this->endpoint->getEndpoint());
		$sparql->setPrefixes($this->endpoint->getPrefixes());
		try {
			$query = "SELECT * WHERE { <".$uri."> ?b ?c }";
			$sparql->query($query);
			$rows = $sparql->getNumRows();
			return $rows > 0;
		} catch (Exception $ex) {
			if ($sparql->isAlive()) throw $ex;
			// otherwise, do nothing, since it surely is temporarily down
		}
	}
	
	public function getSameAs($author) {
		$sameas = array();
		$e = Endpoint_Manager::getEndpoint($author, false);
		$sparql = new Sparql($e->getEndpoint());
		$sparql->setPrefixes($e->getPrefixes());
		try {
			$query = "SELECT ?sameas WHERE { <".$author."> owl:sameAs ?sameas }";
			$sparql->query($query);
			while ($row = $sparql->getNextRow())
				$sameas[] = $row[0];
		} catch (Exception $ex) {
			if ($sparql->isAlive()) throw $ex;
			// otherwise, do nothing, since it surely is temporarily down
		} 
		return $sameas;
	}
	
	public function getEquivalents($author) {		
		$e = Endpoint_Manager::getEndpoints();
		unset($e[array_search(Endpoint_Manager::belongs($author), $e)]);

		$f = array();
		
		$sa = $this->getSameAs($author);
		$sa[] = $author;
		$s = array();
		foreach ($e as $k => $v) $s[$k] = $sa;
		
		while (array_reduce(array_map(function ($a) { return count($a) != 0; }, $s), function($a,$b) { return $a||$b; }, false)) {
			foreach ($e as $k => $v) {
				if (count($s[$k]) == 0) continue;
				foreach ($s[$k] as $u) {
					if (Endpoint_Manager::belongs($u, false) == $v->getEndpoint()) {
						$f[] = $u;
						unset($e[$k]);
						unset($s[$k]);
						continue 2;
					}
				}
				$mediator = new Mediator($e[$k]->getEndpoint());
				$uri = $mediator->getDirectEquivalent($s[$k]);
				if ($uri) {
					$f[] = $uri;
					$sa = $mediator->getSameAs($uri);
					foreach ($s as $k_s => $v_s) {
						$s[$k_s] = array_merge($s[$k_s], $sa); 
					}
					unset($e[$k]);
					unset($s[$k]);
				} else $s[$k] = array();
			}
		}
		
		return $f;
	}
	
}
?>
